schedule = {}
schedule["Kitchen"] = {
    "DailySchedules": [
        {
            "DayOfWeek": 0,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 1,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 2,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 3,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 4,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 5,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 6,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "23:00:00"},
            ],
        },
    ]
}
schedule["Basement One"] = {
    "DailySchedules": [
        {
            "DayOfWeek": 0,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 1,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 2,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 3,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 4,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 5,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 6,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
    ]
}
schedule["Dining room"] = {
    "DailySchedules": [
        {
            "DayOfWeek": 0,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 1,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 2,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 3,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 4,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 5,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 6,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
    ]
}
schedule["Landing"] = {
    "DailySchedules": [
        {
            "DayOfWeek": 0,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 1,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 2,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 3,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 4,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 5,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 6,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
    ]
}
schedule["Guest Bedroom"] = {
    "DailySchedules": [
        {
            "DayOfWeek": 0,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 1,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 2,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 3,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 4,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 5,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 6,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "20:00:00"},
            ],
        },
    ]
}
schedule["Sues Bedroom"] = {
    "DailySchedules": [
        {
            "DayOfWeek": 0,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 1,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 2,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 3,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 4,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 5,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 6,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "08:30:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
    ]
}
schedule["Matts bedroom"] = {
    "DailySchedules": [
        {
            "DayOfWeek": 0,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 1,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 2,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 3,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 4,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 5,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
        {
            "DayOfWeek": 6,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 20.0, "TimeOfDay": "20:00:00"},
            ],
        },
    ]
}
schedule["Basement Two"] = {
    "DailySchedules": [
        {
            "DayOfWeek": 0,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 1,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 2,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 3,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 4,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 5,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
        {
            "DayOfWeek": 6,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "06:00:00"}],
        },
    ]
}
schedule["Bathroom"] = {
    "DailySchedules": [
        {
            "DayOfWeek": 0,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "18:00:00"},
            ],
        },
        {
            "DayOfWeek": 1,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "18:00:00"},
            ],
        },
        {
            "DayOfWeek": 2,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "18:00:00"},
            ],
        },
        {
            "DayOfWeek": 3,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "18:00:00"},
            ],
        },
        {
            "DayOfWeek": 4,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "18:00:00"},
            ],
        },
        {
            "DayOfWeek": 5,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "18:00:00"},
            ],
        },
        {
            "DayOfWeek": 6,
            "Switchpoints": [
                {"heatSetpoint": 14.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "18:00:00"},
            ],
        },
    ]
}
schedule["Dressing Room"] = {
    "DailySchedules": [
        {
            "DayOfWeek": 0,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
            ],
        },
        {
            "DayOfWeek": 1,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
            ],
        },
        {
            "DayOfWeek": 2,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
            ],
        },
        {
            "DayOfWeek": 3,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
            ],
        },
        {
            "DayOfWeek": 4,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
            ],
        },
        {
            "DayOfWeek": 5,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
            ],
        },
        {
            "DayOfWeek": 6,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 19.0, "TimeOfDay": "06:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "09:00:00"},
            ],
        },
    ]
}
schedule["Sitting Room"] = {
    "DailySchedules": [
        {
            "DayOfWeek": 0,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 1,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 2,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 3,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 4,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "09:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 5,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
        {
            "DayOfWeek": 6,
            "Switchpoints": [
                {"heatSetpoint": 19.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "17:30:00"},
                {"heatSetpoint": 14.0, "TimeOfDay": "23:00:00"},
            ],
        },
    ]
}
schedule["Hall"] = {
    "DailySchedules": [
        {
            "DayOfWeek": 0,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"}],
        },
        {
            "DayOfWeek": 1,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"}],
        },
        {
            "DayOfWeek": 2,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"}],
        },
        {
            "DayOfWeek": 3,
            "Switchpoints": [
                {"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "06:30:00"},
                {"heatSetpoint": 18.0, "TimeOfDay": "08:00:00"},
                {"heatSetpoint": 21.0, "TimeOfDay": "18:00:00"},
                {"heatSetpoint": 16.0, "TimeOfDay": "22:30:00"},
            ],
        },
        {
            "DayOfWeek": 4,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"}],
        },
        {
            "DayOfWeek": 5,
            "Switchpoints": [{"heatSetpoint": 16.0, "TimeOfDay": "00:00:00"}],
        },
    ]
}
